@@ -32,6 +32,24 @@ ignored_files = {
   "rngparser.c": "not yet integrated",
   "rngparser.h": "not yet integrated",
   "elfgcchack.h": "not a normal header",
+  "testHTML.c": "test tool",
+  "testReader.c": "test tool",
+  "testSchemas.c": "test tool",
+  "testXPath.c": "test tool",
+  "testAutomata.c": "test tool",
+  "testModule.c": "test tool",
+  "testRegexp.c": "test tool",
+  "testThreads.c": "test tool",
+  "testC14N.c": "test tool",
+  "testRelax.c": "test tool",
+  "testThreadsWin32.c": "test tool",
+  "testSAX.c": "test tool",
+  "testURI.c": "test tool",
+  "testapi.c": "generated regression tests",
+  "runtest.c": "regression tests program",
+  "runsuite.c": "regression tests program",
+  "tst.c": "not part of the library",
+  "testdso.c": "test for dynamid shared libraries",
 }
 
 ignored_words = {
@@ -45,6 +63,7 @@ ignored_words = {
   "EXSLTPUBFUN": (0, "Special macro for extern funcs for win32"),
   "XMLCALL": (0, "Special macro for win32 calls"),
   "XSLTCALL": (0, "Special macro for win32 calls"),
+  "XMLCDECL": (0, "Special macro for win32 calls"),
   "EXSLTCALL": (0, "Special macro for win32 calls"),
   "__declspec": (3, "Windows keyword"),
   "__stdcall": (0, "Windows keyword"),
@@ -68,9 +87,10 @@ def uniq(items):
     return d.keys()
 
 class identifier:
-    def __init__(self, name, module=None, type=None, lineno = 0,
+    def __init__(self, name, header=None, module=None, type=None, lineno = 0,
                  info=None, extra=None, conditionals = None):
         self.name = name
+	self.header = header
 	self.module = module
 	self.type = type
 	self.info = info
@@ -100,6 +120,8 @@ class identifier:
 	return r
 
 
+    def set_header(self, header):
+        self.header = header
     def set_module(self, module):
         self.module = module
     def set_type(self, type):
@@ -120,6 +142,8 @@ class identifier:
 
     def get_name(self):
         return self.name
+    def get_header(self):
+        return self.module
     def get_module(self):
         return self.module
     def get_type(self):
@@ -135,12 +159,14 @@ class identifier:
     def get_conditionals(self):
         return self.conditionals
 
-    def update(self, module, type = None, info = None, extra=None,
+    def update(self, header, module, type = None, info = None, extra=None,
                conditionals=None):
 	if self.name == debugsym:
 	    print "=> update %s : %s" % (debugsym, (module, type, info,
 	                                 extra, conditionals))
-        if module != None and self.module == None:
+        if header != None and self.header == None:
+	    self.set_header(module)
+        if module != None and (self.module == None or self.header == self.module):
 	    self.set_module(module)
         if type != None and self.type == None:
 	    self.set_type(type)
@@ -151,7 +177,6 @@ class identifier:
         if conditionals != None:
 	    self.set_conditionals(conditionals)
 
-
 class index:
     def __init__(self, name = "noname"):
         self.name = name
@@ -166,15 +191,15 @@ class index:
 	self.references = {}
 	self.info = {}
 
-    def add_ref(self, name, module, static, type, lineno, info=None, extra=None, conditionals = None):
+    def add_ref(self, name, header, module, static, type, lineno, info=None, extra=None, conditionals = None):
         if name[0:2] == '__':
 	    return None
         d = None
         try:
 	   d = self.identifiers[name]
-	   d.update(module, type, lineno, info, extra, conditionals)
+	   d.update(header, module, type, lineno, info, extra, conditionals)
 	except:
-	   d = identifier(name, module, type, lineno, info, extra, conditionals)
+	   d = identifier(name, header, module, type, lineno, info, extra, conditionals)
 	   self.identifiers[name] = d
 
 	if d != None and static == 1:
@@ -188,15 +213,15 @@ class index:
 
 	return d
 
-    def add(self, name, module, static, type, lineno, info=None, extra=None, conditionals = None):
+    def add(self, name, header, module, static, type, lineno, info=None, extra=None, conditionals = None):
         if name[0:2] == '__':
 	    return None
         d = None
         try:
 	   d = self.identifiers[name]
-	   d.update(module, type, lineno, info, extra, conditionals)
+	   d.update(header, module, type, lineno, info, extra, conditionals)
 	except:
-	   d = identifier(name, module, type, lineno, info, extra, conditionals)
+	   d = identifier(name, header, module, type, lineno, info, extra, conditionals)
 	   self.identifiers[name] = d
 
 	if d != None and static == 1:
@@ -237,7 +262,7 @@ class index:
 	         del self.macros[id]
 	     if self.functions.has_key(id):
 	         print "function %s from %s redeclared in %s" % (
-		    id, self.functions[id].module, idx.functions[id].module)
+		    id, self.functions[id].header, idx.functions[id].header)
 	     else:
 	         self.functions[id] = idx.functions[id]
 		 self.identifiers[id] = idx.functions[id]
@@ -250,21 +275,21 @@ class index:
 	         del self.macros[id]
 	     if self.variables.has_key(id):
 	         print "variable %s from %s redeclared in %s" % (
-		    id, self.variables[id].module, idx.variables[id].module)
+		    id, self.variables[id].header, idx.variables[id].header)
 	     else:
 	         self.variables[id] = idx.variables[id]
 		 self.identifiers[id] = idx.variables[id]
         for id in idx.structs.keys():
 	     if self.structs.has_key(id):
 	         print "struct %s from %s redeclared in %s" % (
-		    id, self.structs[id].module, idx.structs[id].module)
+		    id, self.structs[id].header, idx.structs[id].header)
 	     else:
 	         self.structs[id] = idx.structs[id]
 		 self.identifiers[id] = idx.structs[id]
         for id in idx.typedefs.keys():
 	     if self.typedefs.has_key(id):
 	         print "typedef %s from %s redeclared in %s" % (
-		    id, self.typedefs[id].module, idx.typedefs[id].module)
+		    id, self.typedefs[id].header, idx.typedefs[id].header)
 	     else:
 	         self.typedefs[id] = idx.typedefs[id]
 		 self.identifiers[id] = idx.typedefs[id]
@@ -281,14 +306,14 @@ class index:
                  continue
 	     if self.macros.has_key(id):
 	         print "macro %s from %s redeclared in %s" % (
-		    id, self.macros[id].module, idx.macros[id].module)
+		    id, self.macros[id].header, idx.macros[id].header)
 	     else:
 	         self.macros[id] = idx.macros[id]
 		 self.identifiers[id] = idx.macros[id]
         for id in idx.enums.keys():
 	     if self.enums.has_key(id):
 	         print "enum %s from %s redeclared in %s" % (
-		    id, self.enums[id].module, idx.enums[id].module)
+		    id, self.enums[id].header, idx.enums[id].header)
 	     else:
 	         self.enums[id] = idx.enums[id]
 		 self.identifiers[id] = idx.enums[id]
@@ -296,11 +321,18 @@ class index:
     def merge_public(self, idx):
         for id in idx.functions.keys():
 	     if self.functions.has_key(id):
+	         # check that function condition agrees with header
+	         if idx.functions[id].conditionals != \
+		    self.functions[id].conditionals:
+		     print "Header condition differs from Function for %s:" \
+		        % id
+		     print "  H: %s" % self.functions[id].conditionals
+		     print "  C: %s" % idx.functions[id].conditionals
 	         up = idx.functions[id]
-	         self.functions[id].update(None, up.type, up.info, up.extra)
+	         self.functions[id].update(None, up.module, up.type, up.info, up.extra)
 	 #     else:
 	 #         print "Function %s from %s is not declared in headers" % (
-	#	        id, idx.functions[id].module)
+	 #	        id, idx.functions[id].module)
 	 # TODO: do the same for variables.
 
     def analyze_dict(self, type, dict):
@@ -565,13 +597,21 @@ class CParser:
         return self.lexer.getlineno()
 
     def index_add(self, name, module, static, type, info=None, extra = None):
-        self.index.add(name, module, static, type, self.lineno(),
-	               info, extra, self.conditionals)
+	if self.is_header == 1:
+	    self.index.add(name, module, module, static, type, self.lineno(),
+			   info, extra, self.conditionals)
+	else:
+	    self.index.add(name, None, module, static, type, self.lineno(),
+			   info, extra, self.conditionals)
 
     def index_add_ref(self, name, module, static, type, info=None,
                       extra = None):
-        self.index.add_ref(name, module, static, type, self.lineno(),
-	               info, extra, self.conditionals)
+	if self.is_header == 1:
+	    self.index.add_ref(name, module, module, static, type,
+	                       self.lineno(), info, extra, self.conditionals)
+	else:
+	    self.index.add_ref(name, None, module, static, type, self.lineno(),
+			       info, extra, self.conditionals)
 
     def warning(self, msg):
         if self.no_error:
@@ -813,12 +853,12 @@ class CParser:
 	    i = 0
 	    while i < nbargs:
 	        if args[i][2] == None and args[i][0] != "void" and args[i][1] != None:
-		    self.warning("Function comment for %s lack description of arg %s" % (name, args[i][1]))
+		    self.warning("Function comment for %s lacks description of arg %s" % (name, args[i][1]))
 		i = i + 1
 	    if retdesc == "" and ret[0] != "void":
-		self.warning("Function comment for %s lack description of return value" % (name))
+		self.warning("Function comment for %s lacks description of return value" % (name))
 	    if desc == "":
-	        self.warning("Function comment for %s lack description of the function" % (name))
+	        self.warning("Function comment for %s lacks description of the function" % (name))
 
 
 	return(((ret[0], retdesc), args, desc))
@@ -857,22 +897,61 @@ class CParser:
 		self.index_add(name, self.filename, not self.is_header,
 		                "macro", info)
 		return token
-	if (name == "#ifdef" or name == "#ifndef") and self.is_header:
+
+	#
+	# Processing of conditionals modified by Bill 1/1/05
+	#
+	# We process conditionals (i.e. tokens from #ifdef, #ifndef,
+	# #if, #else and #endif) for headers and mainline code,
+	# store the ones from the header in libxml2-api.xml, and later
+	# (in the routine merge_public) verify that the two (header and
+	# mainline code) agree.
+	#
+	# There is a small problem with processing the headers. Some of
+	# the variables are not concerned with enabling / disabling of
+	# library functions (e.g. '__XML_PARSER_H__'), and we don't want
+	# them to be included in libxml2-api.xml, or involved in
+	# the check between the header and the mainline code.  To
+	# accomplish this, we ignore any conditional which doesn't include
+	# the string 'ENABLED'
+	#
+	if name == "#ifdef":
+	    apstr = self.lexer.tokens[0][1]
 	    try:
-	        self.defines.append(self.lexer.tokens[0][1])
-	        if string.find(self.lexer.tokens[0][1], 'ENABLED') != -1:
-		    self.conditionals.append(self.lexer.tokens[0][1])
-#		print self.conditionals
-#		print self.defines
+	        self.defines.append(apstr)
+		if string.find(apstr, 'ENABLED') != -1:
+		    self.conditionals.append("defined(%s)" % apstr)    
 	    except:
 	        pass
-	if name == "#endif" and self.is_header:
-	    if self.conditionals != [] and self.defines != [] and \
-	       self.defines[-1] == self.conditionals[-1]:
+	elif name == "#ifndef":
+	    apstr = self.lexer.tokens[0][1]
+	    try:
+	        self.defines.append(apstr)
+		if string.find(apstr, 'ENABLED') != -1:
+		    self.conditionals.append("!defined(%s)" % apstr)    
+	    except:
+	        pass
+	elif name == "#if":
+	    apstr = ""
+	    for tok in self.lexer.tokens:
+	        if apstr != "":
+		    apstr = apstr + " "
+	        apstr = apstr + tok[1]
+	    try:
+	        self.defines.append(apstr)
+		if string.find(apstr, 'ENABLED') != -1:
+		    self.conditionals.append(apstr)
+	    except:
+	        pass
+	elif name == "#else":
+	    if self.conditionals != [] and \
+	       string.find(self.defines[-1], 'ENABLED') != -1:
+	        self.conditionals[-1] = "!(%s)" % self.conditionals[-1]
+	elif name == "#endif":
+	    if self.conditionals != [] and \
+	       string.find(self.defines[-1], 'ENABLED') != -1:
 	        self.conditionals = self.conditionals[:-1]
 	    self.defines = self.defines[:-1]
-#	    print self.defines
-#	    print self.conditionals
 	token = self.lexer.token()
 	while token != None and token[0] == 'preproc' and \
 	    token[1][0] != '#':
@@ -895,6 +974,14 @@ class CParser:
 	    elif token[0] == 'preproc':
 		token = self.parsePreproc(token)
 		continue
+	    elif token[0] == "name" and token[1] == "__const":
+	        token = ("name", "const")
+		return token
+	    elif token[0] == "name" and token[1] == "__attribute":
+		token = self.lexer.token()
+		while token != None and token[1] != ";":
+		    token = self.lexer.token()
+		return token
 	    elif token[0] == "name" and ignored_words.has_key(token[1]):
 	        (n, info) = ignored_words[token[1]]
 		i = 0
@@ -1130,7 +1217,9 @@ class CParser:
 	    return token
 
 	while token[0] == "name" and ( 
-	      token[1] == "const" or token[1] == "unsigned"):
+	      token[1] == "const" or \
+	      token[1] == "unsigned" or \
+	      token[1] == "signed"):
 	    if self.type == "":
 	        self.type = token[1]
 	    else:
@@ -1555,12 +1644,14 @@ class docBuilder:
         module = os.path.basename(file)
 	if module[-2:] == '.h':
 	    module = module[:-2]
+	elif module[-2:] == '.c':
+	    module = module[:-2]
 	return module
 
     def serialize_enum(self, output, name):
         id = self.idx.enums[name]
         output.write("    <enum name='%s' file='%s'" % (name,
-	             self.modulename_file(id.module)))
+	             self.modulename_file(id.header)))
 	if id.info != None:
 	    info = id.info
 	    if info[0] != None and info[0] != '':
@@ -1578,7 +1669,7 @@ class docBuilder:
     def serialize_macro(self, output, name):
         id = self.idx.macros[name]
         output.write("    <macro name='%s' file='%s'>\n" % (name,
-	             self.modulename_file(id.module)))
+	             self.modulename_file(id.header)))
 	if id.info != None:
             try:
 		(args, desc) = id.info
@@ -1601,7 +1692,7 @@ class docBuilder:
         id = self.idx.typedefs[name]
 	if id.info[0:7] == 'struct ':
 	    output.write("    <struct name='%s' file='%s' type='%s'" % (
-	             name, self.modulename_file(id.module), id.info))
+	             name, self.modulename_file(id.header), id.info))
 	    name = id.info[7:]
 	    if self.idx.structs.has_key(name) and ( \
 	       type(self.idx.structs[name].info) == type(()) or
@@ -1623,27 +1714,35 @@ class docBuilder:
 	        output.write("/>\n");
 	else :
 	    output.write("    <typedef name='%s' file='%s' type='%s'/>\n" % (
-	             name, self.modulename_file(id.module), id.info))
+	             name, self.modulename_file(id.header), id.info))
 
     def serialize_variable(self, output, name):
         id = self.idx.variables[name]
 	if id.info != None:
 	    output.write("    <variable name='%s' file='%s' type='%s'/>\n" % (
-		    name, self.modulename_file(id.module), id.info))
+		    name, self.modulename_file(id.header), id.info))
 	else:
 	    output.write("    <variable name='%s' file='%s'/>\n" % (
-	            name, self.modulename_file(id.module)))
+	            name, self.modulename_file(id.header)))
 	              
     def serialize_function(self, output, name):
         id = self.idx.functions[name]
 	if name == debugsym:
 	    print "=>", id
 
-        output.write("    <%s name='%s' file='%s'>\n" % (id.type, name,
-	             self.modulename_file(id.module)))
+        output.write("    <%s name='%s' file='%s' module='%s'>\n" % (id.type,
+	             name, self.modulename_file(id.header),
+		     self.modulename_file(id.module)))
+	#
+	# Processing of conditionals modified by Bill 1/1/05
+	#
 	if id.conditionals != None:
+	    apstr = ""
 	    for cond in id.conditionals:
-	        output.write("      <cond>%s</cond>\n"% (cond));
+	        if apstr != "":
+		    apstr = apstr + " &amp;&amp; "
+		apstr = apstr + cond
+	    output.write("      <cond>%s</cond>\n"% (apstr));
 	try:
 	    (ret, params, desc) = id.info
 	    output.write("      <info>%s</info>\n" % (escape(desc)))
@@ -1815,7 +1914,7 @@ class docBuilder:
 	typ.sort()
 	for id in typ:
 	    idf = self.idx.identifiers[id]
-	    module = idf.module
+	    module = idf.header
 	    output.write("    <reference name='%s' href='%s'/>\n" % (id,
 	                 'html/' + self.basename + '-' +
 		         self.modulename_file(module) + '.html#' +
