@@ -70,8 +70,10 @@ typedef xmlEntitiesTable *xmlEntitiesTablePtr;
  * External functions:
  */
 
+#ifdef LIBXML_LEGACY_ENABLED
 XMLPUBFUN void XMLCALL		
 		xmlInitializePredefinedEntities	(void);
+#endif /* LIBXML_LEGACY_ENABLED */
 XMLPUBFUN xmlEntityPtr XMLCALL
 			xmlAddDocEntity		(xmlDocPtr doc,
 						 const xmlChar *name,
@@ -97,9 +99,11 @@ XMLPUBFUN xmlEntityPtr XMLCALL
 XMLPUBFUN xmlEntityPtr XMLCALL		
 			xmlGetParameterEntity	(xmlDocPtr doc,
 						 const xmlChar *name);
+#ifdef LIBXML_LEGACY_ENABLED
 XMLPUBFUN const xmlChar * XMLCALL		
 			xmlEncodeEntities	(xmlDocPtr doc,
 						 const xmlChar *input);
+#endif /* LIBXML_LEGACY_ENABLED */
 XMLPUBFUN xmlChar * XMLCALL		
 			xmlEncodeEntitiesReentrant(xmlDocPtr doc,
 						 const xmlChar *input);
@@ -108,8 +112,10 @@ XMLPUBFUN xmlChar * XMLCALL
 						 const xmlChar *input);
 XMLPUBFUN xmlEntitiesTablePtr XMLCALL	
 			xmlCreateEntitiesTable	(void);
+#ifdef LIBXML_TREE_ENABLED
 XMLPUBFUN xmlEntitiesTablePtr XMLCALL	
 			xmlCopyEntitiesTable	(xmlEntitiesTablePtr table);
+#endif /* LIBXML_TREE_ENABLED */
 XMLPUBFUN void XMLCALL			
 			xmlFreeEntitiesTable	(xmlEntitiesTablePtr table);
 #ifdef LIBXML_OUTPUT_ENABLED
@@ -120,8 +126,10 @@ XMLPUBFUN void XMLCALL
 			xmlDumpEntityDecl	(xmlBufferPtr buf,
 						 xmlEntityPtr ent);
 #endif /* LIBXML_OUTPUT_ENABLED */
+#ifdef LIBXML_LEGACY_ENABLED
 XMLPUBFUN void XMLCALL			
 			xmlCleanupPredefinedEntities(void);
+#endif /* LIBXML_LEGACY_ENABLED */
 
 
 #ifdef __cplusplus
