@@ -44,15 +44,15 @@ typedef enum {
     XML_SCHEMAS_TOKEN,
     XML_SCHEMAS_LANGUAGE,
     XML_SCHEMAS_NMTOKEN,
-    XML_SCHEMAS_NMTOKENS, 
+    XML_SCHEMAS_NMTOKENS,
     XML_SCHEMAS_NAME,
     XML_SCHEMAS_QNAME,
     XML_SCHEMAS_NCNAME,
     XML_SCHEMAS_ID,
     XML_SCHEMAS_IDREF,
-    XML_SCHEMAS_IDREFS, 
+    XML_SCHEMAS_IDREFS,
     XML_SCHEMAS_ENTITY,
-    XML_SCHEMAS_ENTITIES, 
+    XML_SCHEMAS_ENTITIES,
     XML_SCHEMAS_NOTATION,
     XML_SCHEMAS_ANYURI,
     XML_SCHEMAS_INTEGER,
@@ -71,7 +71,7 @@ typedef enum {
     XML_SCHEMAS_HEXBINARY,
     XML_SCHEMAS_BASE64BINARY,
     XML_SCHEMAS_ANYTYPE,
-    XML_SCHEMAS_ANYSIMPLETYPE   
+    XML_SCHEMAS_ANYSIMPLETYPE
 } xmlSchemaValType;
 
 /*
@@ -83,7 +83,7 @@ typedef enum {
     XML_SCHEMA_TYPE_FACET,
     XML_SCHEMA_TYPE_SIMPLE,
     XML_SCHEMA_TYPE_COMPLEX,
-    XML_SCHEMA_TYPE_SEQUENCE,
+    XML_SCHEMA_TYPE_SEQUENCE = 6,
     XML_SCHEMA_TYPE_CHOICE,
     XML_SCHEMA_TYPE_ALL,
     XML_SCHEMA_TYPE_SIMPLE_CONTENT,
@@ -99,6 +99,10 @@ typedef enum {
     XML_SCHEMA_TYPE_LIST,
     XML_SCHEMA_TYPE_UNION,
     XML_SCHEMA_TYPE_ANY_ATTRIBUTE,
+    XML_SCHEMA_TYPE_IDC_UNIQUE,
+    XML_SCHEMA_TYPE_IDC_KEY,
+    XML_SCHEMA_TYPE_IDC_KEYREF,
+    XML_SCHEMA_TYPE_PARTICLE = 25, 
     XML_SCHEMA_FACET_MININCLUSIVE = 1000,
     XML_SCHEMA_FACET_MINEXCLUSIVE,
     XML_SCHEMA_FACET_MAXINCLUSIVE,
@@ -110,7 +114,8 @@ typedef enum {
     XML_SCHEMA_FACET_WHITESPACE,
     XML_SCHEMA_FACET_LENGTH,
     XML_SCHEMA_FACET_MAXLENGTH,
-    XML_SCHEMA_FACET_MINLENGTH
+    XML_SCHEMA_FACET_MINLENGTH,
+    XML_SCHEMA_EXTRA_QNAMEREF = 2000
 } xmlSchemaTypeType;
 
 typedef enum {
@@ -149,25 +154,25 @@ struct _xmlSchemaAnnot {
  * Skip unknown attribute from validation
  * Obsolete, not used anymore.
  */
-#define XML_SCHEMAS_ANYATTR_SKIP	1
+#define XML_SCHEMAS_ANYATTR_SKIP        1
 /**
  * XML_SCHEMAS_ANYATTR_LAX:
  *
  * Ignore validation non definition on attributes
  * Obsolete, not used anymore.
  */
-#define XML_SCHEMAS_ANYATTR_LAX		2
+#define XML_SCHEMAS_ANYATTR_LAX                2
 /**
  * XML_SCHEMAS_ANYATTR_STRICT:
  *
  * Apply strict validation rules on attributes
  * Obsolete, not used anymore.
  */
-#define XML_SCHEMAS_ANYATTR_STRICT	3
+#define XML_SCHEMAS_ANYATTR_STRICT        3
 /**
  * XML_SCHEMAS_ANY_SKIP:
  *
- * Skip unknown attribute from validation 
+ * Skip unknown attribute from validation
  */
 #define XML_SCHEMAS_ANY_SKIP        1
 /**
@@ -204,7 +209,7 @@ struct _xmlSchemaAnnot {
  */
 #define XML_SCHEMAS_ATTR_USE_OPTIONAL 2
 /**
- * XML_SCHEMAS_ATTR_GLOABAL:
+ * XML_SCHEMAS_ATTR_GLOBAL:
  *
  * allow elements in no namespace
  */
@@ -214,20 +219,20 @@ struct _xmlSchemaAnnot {
  *
  * allow elements in no namespace
  */
-#define XML_SCHEMAS_ATTR_NSDEFAULT	1 << 7
+#define XML_SCHEMAS_ATTR_NSDEFAULT        1 << 7
 /**
- * XML_SCHEMAS_ATTR_NSDEFAULT:
+ * XML_SCHEMAS_ATTR_INTERNAL_RESOLVED:
  *
- * this is set when the "type" and "ref" references 
+ * this is set when the "type" and "ref" references
  * have been resolved.
  */
-#define XML_SCHEMAS_ATTR_INTERNAL_RESOLVED	1 << 8
+#define XML_SCHEMAS_ATTR_INTERNAL_RESOLVED        1 << 8
 /**
  * XML_SCHEMAS_ATTR_FIXED:
  *
  * the attribute has a fixed value
  */
-#define XML_SCHEMAS_ATTR_FIXED	1 << 9
+#define XML_SCHEMAS_ATTR_FIXED        1 << 9
 
 /**
  * xmlSchemaAttribute:
@@ -237,8 +242,8 @@ struct _xmlSchemaAnnot {
 typedef struct _xmlSchemaAttribute xmlSchemaAttribute;
 typedef xmlSchemaAttribute *xmlSchemaAttributePtr;
 struct _xmlSchemaAttribute {
-    xmlSchemaTypeType type;	/* The kind of type */
-    struct _xmlSchemaAttribute *next;/* the next attribute if in a group ... */ 
+    xmlSchemaTypeType type;        /* The kind of type */
+    struct _xmlSchemaAttribute *next;/* the next attribute if in a group ... */
     const xmlChar *name; /* name of the declaration or empty if particle */
     const xmlChar *id;
     const xmlChar *ref; /* the local name of the attribute decl. if a particle */
@@ -252,9 +257,11 @@ struct _xmlSchemaAttribute {
     const xmlChar *defValue;
     xmlSchemaTypePtr subtypes; /* the type definition */
     xmlNodePtr node;
-    const xmlChar *targetNamespace; 
+    const xmlChar *targetNamespace;
     int flags;
     const xmlChar *refPrefix;
+    xmlSchemaValPtr defVal;
+    xmlSchemaAttributePtr refDecl;
 };
 
 /**
@@ -318,6 +325,12 @@ struct _xmlSchemaWildcard {
  * The attribute wildcard has been already builded.
  */
 #define XML_SCHEMAS_ATTRGROUP_GLOBAL 1 << 1
+/**
+ * XML_SCHEMAS_ATTRGROUP_MARKED:
+ *
+ * Marks the attr group as marked; used for circular checks.
+ */
+#define XML_SCHEMAS_ATTRGROUP_MARKED 1 << 2
 
 /**
  * An attribute group definition.
@@ -328,7 +341,7 @@ struct _xmlSchemaWildcard {
 typedef struct _xmlSchemaAttributeGroup xmlSchemaAttributeGroup;
 typedef xmlSchemaAttributeGroup *xmlSchemaAttributeGroupPtr;
 struct _xmlSchemaAttributeGroup {
-    xmlSchemaTypeType type;	/* The kind of type */
+    xmlSchemaTypeType type;        /* The kind of type */
     struct _xmlSchemaAttribute *next;/* the next attribute if in a group ... */
     const xmlChar *name;
     const xmlChar *id;
@@ -341,6 +354,8 @@ struct _xmlSchemaAttributeGroup {
     int flags;
     xmlSchemaWildcardPtr attributeWildcard;
     const xmlChar *refPrefix;
+    xmlSchemaAttributeGroupPtr refItem; /* The referenced attribute group */
+    const xmlChar *targetNamespace;
 };
 
 /**
@@ -371,7 +386,7 @@ struct _xmlSchemaFacetLink {
  *
  * the element content type is mixed
  */
-#define XML_SCHEMAS_TYPE_MIXED		1 << 0
+#define XML_SCHEMAS_TYPE_MIXED                1 << 0
 /**
  * XML_SCHEMAS_TYPE_DERIVATION_METHOD_EXTENSION:
  *
@@ -396,7 +411,7 @@ struct _xmlSchemaFacetLink {
  * the complexType owns an attribute wildcard, i.e.
  * it can be freed by the complexType
  */
-#define XML_SCHEMAS_TYPE_OWNED_ATTR_WILDCARD    1 << 4
+#define XML_SCHEMAS_TYPE_OWNED_ATTR_WILDCARD    1 << 4 /* Obsolete. */
 /**
  * XML_SCHEMAS_TYPE_VARIETY_ABSENT:
  *
@@ -446,17 +461,96 @@ struct _xmlSchemaFacetLink {
  */
 #define XML_SCHEMAS_TYPE_FINAL_UNION    1 << 12
 /**
- * XML_SCHEMAS_TYPE_FINAL_UNION:
+ * XML_SCHEMAS_TYPE_FINAL_DEFAULT:
  *
- * the simpleType has a final of "union".
+ * the simpleType has a final of "default".
  */
 #define XML_SCHEMAS_TYPE_FINAL_DEFAULT    1 << 13
 /**
- * XML_SCHEMAS_TYPE_FINAL_UNION:
+ * XML_SCHEMAS_TYPE_BUILTIN_PRIMITIVE:
  *
- * the simpleType has a final of "union".
+ * Marks the item as a builtin primitive.
  */
 #define XML_SCHEMAS_TYPE_BUILTIN_PRIMITIVE    1 << 14
+/**
+ * XML_SCHEMAS_TYPE_MARKED:
+ *
+ * Marks the item as marked; used for circular checks.
+ */
+#define XML_SCHEMAS_TYPE_MARKED        1 << 16
+/**
+ * XML_SCHEMAS_TYPE_BLOCK_DEFAULT:
+ *
+ * the complexType did not specify 'block' so use the default of the
+ * <schema> item.
+ */
+#define XML_SCHEMAS_TYPE_BLOCK_DEFAULT    1 << 17
+/**
+ * XML_SCHEMAS_TYPE_BLOCK_EXTENSION:
+ *
+ * the complexType has a 'block' of "extension".
+ */
+#define XML_SCHEMAS_TYPE_BLOCK_EXTENSION    1 << 18
+/**
+ * XML_SCHEMAS_TYPE_BLOCK_RESTRICTION:
+ *
+ * the complexType has a 'block' of "restriction".
+ */
+#define XML_SCHEMAS_TYPE_BLOCK_RESTRICTION    1 << 19
+/**
+ * XML_SCHEMAS_TYPE_ABSTRACT:
+ *
+ * the simple/complexType is abstract.
+ */
+#define XML_SCHEMAS_TYPE_ABSTRACT    1 << 20
+/**
+ * XML_SCHEMAS_TYPE_FACETSNEEDVALUE:
+ *
+ * indicates if the facets need a computed value
+ */
+#define XML_SCHEMAS_TYPE_FACETSNEEDVALUE    1 << 21
+/**
+ * XML_SCHEMAS_TYPE_INTERNAL_RESOLVED:
+ *
+ * indicates that the type was typefixed
+ */
+#define XML_SCHEMAS_TYPE_INTERNAL_RESOLVED    1 << 22
+/**
+ * XML_SCHEMAS_TYPE_INTERNAL_INVALID:
+ *
+ * indicates that the type is invalid
+ */
+#define XML_SCHEMAS_TYPE_INTERNAL_INVALID    1 << 23
+/**
+ * XML_SCHEMAS_TYPE_WHITESPACE_PRESERVE:
+ *
+ * a whitespace-facet value of "preserve"
+ */
+#define XML_SCHEMAS_TYPE_WHITESPACE_PRESERVE    1 << 24
+/**
+ * XML_SCHEMAS_TYPE_WHITESPACE_REPLACE:
+ *
+ * a whitespace-facet value of "replace"
+ */
+#define XML_SCHEMAS_TYPE_WHITESPACE_REPLACE    1 << 25
+/**
+ * XML_SCHEMAS_TYPE_WHITESPACE_COLLAPSE:
+ *
+ * a whitespace-facet value of "collapse"
+ */
+#define XML_SCHEMAS_TYPE_WHITESPACE_COLLAPSE    1 << 26
+/**
+ * XML_SCHEMAS_TYPE_HAS_FACETS:
+ *
+ * has facets
+ */
+#define XML_SCHEMAS_TYPE_HAS_FACETS    1 << 27
+/**
+ * XML_SCHEMAS_TYPE_NORMVALUENEEDED:
+ *
+ * indicates if the facets (pattern) need a normalized value
+ */
+#define XML_SCHEMAS_TYPE_NORMVALUENEEDED    1 << 28
 
 /**
  * _xmlSchemaType:
@@ -464,7 +558,7 @@ struct _xmlSchemaFacetLink {
  * Schemas type definition.
  */
 struct _xmlSchemaType {
-    xmlSchemaTypeType type;	/* The kind of type */
+    xmlSchemaTypeType type;        /* The kind of type */
     struct _xmlSchemaType *next;/* the next type if in a sequence ... */
     const xmlChar *name;
     const xmlChar *id;
@@ -481,7 +575,7 @@ struct _xmlSchemaType {
     xmlSchemaContentType contentType;
     const xmlChar *base;
     const xmlChar *baseNs;
-    xmlSchemaTypePtr baseType; 
+    xmlSchemaTypePtr baseType;
     xmlSchemaFacetPtr facets;
     struct _xmlSchemaType *redef;/* possible redefinitions for the type */
     int recurse;
@@ -492,6 +586,8 @@ struct _xmlSchemaType {
     xmlSchemaFacetLinkPtr facetSet;
     const xmlChar *refPrefix;
     xmlSchemaTypePtr contentTypeDef;
+    xmlRegexpPtr contModel;
+    const xmlChar *targetNamespace;
 };
 
 /*
@@ -506,112 +602,124 @@ struct _xmlSchemaType {
  *
  * the element is nillable
  */
-#define XML_SCHEMAS_ELEM_NILLABLE	1 << 0
+#define XML_SCHEMAS_ELEM_NILLABLE        1 << 0
 /**
  * XML_SCHEMAS_ELEM_GLOBAL:
  *
  * the element is global
  */
-#define XML_SCHEMAS_ELEM_GLOBAL		1 << 1
+#define XML_SCHEMAS_ELEM_GLOBAL                1 << 1
 /**
  * XML_SCHEMAS_ELEM_DEFAULT:
  *
  * the element has a default value
  */
-#define XML_SCHEMAS_ELEM_DEFAULT	1 << 2
+#define XML_SCHEMAS_ELEM_DEFAULT        1 << 2
 /**
  * XML_SCHEMAS_ELEM_FIXED:
  *
  * the element has a fixed value
  */
-#define XML_SCHEMAS_ELEM_FIXED		1 << 3
+#define XML_SCHEMAS_ELEM_FIXED                1 << 3
 /**
  * XML_SCHEMAS_ELEM_ABSTRACT:
  *
  * the element is abstract
  */
-#define XML_SCHEMAS_ELEM_ABSTRACT	1 << 4
+#define XML_SCHEMAS_ELEM_ABSTRACT        1 << 4
 /**
  * XML_SCHEMAS_ELEM_TOPLEVEL:
  *
  * the element is top level
  * obsolete: use XML_SCHEMAS_ELEM_GLOBAL instead
  */
-#define XML_SCHEMAS_ELEM_TOPLEVEL	1 << 5
+#define XML_SCHEMAS_ELEM_TOPLEVEL        1 << 5
 /**
  * XML_SCHEMAS_ELEM_REF:
  *
  * the element is a reference to a type
  */
-#define XML_SCHEMAS_ELEM_REF		1 << 6
+#define XML_SCHEMAS_ELEM_REF                1 << 6
 /**
  * XML_SCHEMAS_ELEM_NSDEFAULT:
  *
  * allow elements in no namespace
  * Obsolete, not used anymore.
  */
-#define XML_SCHEMAS_ELEM_NSDEFAULT	1 << 7
+#define XML_SCHEMAS_ELEM_NSDEFAULT        1 << 7
 /**
- * XML_SCHEMAS_ELEM_INTERNAL_RESOLVED
+ * XML_SCHEMAS_ELEM_INTERNAL_RESOLVED:
  *
  * this is set when "type", "ref", "substitutionGroup"
  * references have been resolved.
  */
-#define XML_SCHEMAS_ELEM_INTERNAL_RESOLVED	1 << 8
+#define XML_SCHEMAS_ELEM_INTERNAL_RESOLVED        1 << 8
  /**
- * XML_SCHEMAS_ELEM_CIRCULAR
+ * XML_SCHEMAS_ELEM_CIRCULAR:
  *
  * a helper flag for the search of circular references.
  */
-#define XML_SCHEMAS_ELEM_CIRCULAR	1 << 9
+#define XML_SCHEMAS_ELEM_CIRCULAR        1 << 9
 /**
  * XML_SCHEMAS_ELEM_BLOCK_ABSENT:
  *
  * the "block" attribute is absent
  */
-#define XML_SCHEMAS_ELEM_BLOCK_ABSENT	1 << 10
+#define XML_SCHEMAS_ELEM_BLOCK_ABSENT        1 << 10
 /**
  * XML_SCHEMAS_ELEM_BLOCK_EXTENSION:
  *
  * disallowed substitutions are absent
  */
-#define XML_SCHEMAS_ELEM_BLOCK_EXTENSION	1 << 11
+#define XML_SCHEMAS_ELEM_BLOCK_EXTENSION        1 << 11
 /**
  * XML_SCHEMAS_ELEM_BLOCK_RESTRICTION:
  *
  * disallowed substitutions: "restriction"
  */
-#define XML_SCHEMAS_ELEM_BLOCK_RESTRICTION	1 << 12
+#define XML_SCHEMAS_ELEM_BLOCK_RESTRICTION        1 << 12
 /**
  * XML_SCHEMAS_ELEM_BLOCK_SUBSTITUTION:
  *
  * disallowed substitutions: "substituion"
  */
-#define XML_SCHEMAS_ELEM_BLOCK_SUBSTITUTION	1 << 13
+#define XML_SCHEMAS_ELEM_BLOCK_SUBSTITUTION        1 << 13
 /**
  * XML_SCHEMAS_ELEM_FINAL_ABSENT:
  *
  * substitution group exclusions are absent
  */
-#define XML_SCHEMAS_ELEM_FINAL_ABSENT	1 << 14
+#define XML_SCHEMAS_ELEM_FINAL_ABSENT        1 << 14
 /**
- * XML_SCHEMAS_ELEM_BLOCK_EXTENSION:
+ * XML_SCHEMAS_ELEM_FINAL_EXTENSION:
  *
  * substitution group exclusions: "extension"
  */
-#define XML_SCHEMAS_ELEM_FINAL_EXTENSION	1 << 15
+#define XML_SCHEMAS_ELEM_FINAL_EXTENSION        1 << 15
 /**
- * XML_SCHEMAS_ELEM_BLOCK_RESTRICTION:
+ * XML_SCHEMAS_ELEM_FINAL_RESTRICTION:
  *
  * substitution group exclusions: "restriction"
  */
-#define XML_SCHEMAS_ELEM_FINAL_RESTRICTION	1 << 16
-
+#define XML_SCHEMAS_ELEM_FINAL_RESTRICTION        1 << 16
+/**
+ * XML_SCHEMAS_ELEM_SUBST_GROUP_HEAD:
+ *
+ * the declaration is a substitution group head
+ */
+#define XML_SCHEMAS_ELEM_SUBST_GROUP_HEAD        1 << 17
+/**
+ * XML_SCHEMAS_ELEM_INTERNAL_CHECKED:
+ *
+ * this is set when the elem decl has been checked against
+ * all constraints
+ */
+#define XML_SCHEMAS_ELEM_INTERNAL_CHECKED        1 << 18
 
 typedef struct _xmlSchemaElement xmlSchemaElement;
 typedef xmlSchemaElement *xmlSchemaElementPtr;
 struct _xmlSchemaElement {
-    xmlSchemaTypeType type;	/* The kind of type */
+    xmlSchemaTypeType type;        /* The kind of type */
     struct _xmlSchemaType *next;/* the next type if in a sequence ... */
     const xmlChar *name;
     const xmlChar *id;
@@ -632,10 +740,12 @@ struct _xmlSchemaElement {
     const xmlChar *substGroupNs;
     const xmlChar *scope;
     const xmlChar *value;
-    struct _xmlSchemaElement *refDecl; /* the element declaration if a particle */
+    struct _xmlSchemaElement *refDecl; /* This will now be used for the substitution group affiliation */
     xmlRegexpPtr contModel;
     xmlSchemaContentType contentType;
     const xmlChar *refPrefix;
+    xmlSchemaValPtr defVal;
+    void *idcs;
 };
 
 /*
@@ -643,30 +753,30 @@ struct _xmlSchemaElement {
  *
  * unknown facet handling
  */
-#define XML_SCHEMAS_FACET_UNKNOWN	0
+#define XML_SCHEMAS_FACET_UNKNOWN        0
 /*
  * XML_SCHEMAS_FACET_PRESERVE:
  *
  * preserve the type of the facet
  */
-#define XML_SCHEMAS_FACET_PRESERVE	1
+#define XML_SCHEMAS_FACET_PRESERVE        1
 /*
  * XML_SCHEMAS_FACET_REPLACE:
  *
  * replace the type of the facet
  */
-#define XML_SCHEMAS_FACET_REPLACE	2
+#define XML_SCHEMAS_FACET_REPLACE        2
 /*
  * XML_SCHEMAS_FACET_COLLAPSE:
  *
  * collapse the types of the facet
  */
-#define XML_SCHEMAS_FACET_COLLAPSE	3
+#define XML_SCHEMAS_FACET_COLLAPSE        3
 /**
  * A facet definition.
  */
 struct _xmlSchemaFacet {
-    xmlSchemaTypeType type;	/* The kind of type */
+    xmlSchemaTypeType type;        /* The kind of type */
     struct _xmlSchemaFacet *next;/* the next type if in a sequence ... */
     const xmlChar *value;
     const xmlChar *id;
@@ -684,10 +794,11 @@ struct _xmlSchemaFacet {
 typedef struct _xmlSchemaNotation xmlSchemaNotation;
 typedef xmlSchemaNotation *xmlSchemaNotationPtr;
 struct _xmlSchemaNotation {
-    xmlSchemaTypeType type;	/* The kind of type */
+    xmlSchemaTypeType type;        /* The kind of type */
     const xmlChar *name;
     xmlSchemaAnnotPtr annot;
     const xmlChar *identifier;
+    const xmlChar *targetNamespace;
 };
 
 /**
@@ -695,62 +806,62 @@ struct _xmlSchemaNotation {
  *
  * the schema requires qualified elements
  */
-#define XML_SCHEMAS_QUALIF_ELEM		1 << 0
+#define XML_SCHEMAS_QUALIF_ELEM                1 << 0
 /**
  * XML_SCHEMAS_QUALIF_ATTR:
  *
  * the schema requires qualified attributes
  */
-#define XML_SCHEMAS_QUALIF_ATTR	    1 << 1
+#define XML_SCHEMAS_QUALIF_ATTR            1 << 1
 /**
  * XML_SCHEMAS_FINAL_DEFAULT_EXTENSION:
  *
  * the schema has "extension" in the set of finalDefault.
  */
-#define XML_SCHEMAS_FINAL_DEFAULT_EXTENSION	1 << 2
+#define XML_SCHEMAS_FINAL_DEFAULT_EXTENSION        1 << 2
 /**
  * XML_SCHEMAS_FINAL_DEFAULT_RESTRICTION:
  *
  * the schema has "restriction" in the set of finalDefault.
  */
-#define XML_SCHEMAS_FINAL_DEFAULT_RESTRICTION	    1 << 3
+#define XML_SCHEMAS_FINAL_DEFAULT_RESTRICTION            1 << 3
 /**
  * XML_SCHEMAS_FINAL_DEFAULT_LIST:
  *
  * the cshema has "list" in the set of finalDefault.
  */
-#define XML_SCHEMAS_FINAL_DEFAULT_LIST	    1 << 4
+#define XML_SCHEMAS_FINAL_DEFAULT_LIST            1 << 4
 /**
  * XML_SCHEMAS_FINAL_DEFAULT_UNION:
  *
  * the schema has "union" in the set of finalDefault.
  */
-#define XML_SCHEMAS_FINAL_DEFAULT_UNION	    1 << 5
+#define XML_SCHEMAS_FINAL_DEFAULT_UNION            1 << 5
 /**
  * XML_SCHEMAS_BLOCK_DEFAULT_EXTENSION:
  *
  * the schema has "extension" in the set of blockDefault.
  */
-#define XML_SCHEMAS_BLOCK_DEFAULT_EXTENSION	    1 << 6
+#define XML_SCHEMAS_BLOCK_DEFAULT_EXTENSION            1 << 6
 /**
  * XML_SCHEMAS_BLOCK_DEFAULT_RESTRICTION:
  *
  * the schema has "restriction" in the set of blockDefault.
  */
-#define XML_SCHEMAS_BLOCK_DEFAULT_RESTRICTION	    1 << 7
+#define XML_SCHEMAS_BLOCK_DEFAULT_RESTRICTION            1 << 7
 /**
  * XML_SCHEMAS_BLOCK_DEFAULT_SUBSTITUTION:
  *
  * the schema has "substitution" in the set of blockDefault.
  */
-#define XML_SCHEMAS_BLOCK_DEFAULT_SUBSTITUTION	    1 << 8
+#define XML_SCHEMAS_BLOCK_DEFAULT_SUBSTITUTION            1 << 8
 /**
  * XML_SCHEMAS_INCLUDING_CONVERT_NS:
  *
  * the schema is currently including an other schema with
  * no target namespace.
  */
-#define XML_SCHEMAS_INCLUDING_CONVERT_NS	    1 << 9
+#define XML_SCHEMAS_INCLUDING_CONVERT_NS            1 << 9
 /**
  * _xmlSchema:
  *
@@ -773,15 +884,18 @@ struct _xmlSchema {
 
     xmlHashTablePtr schemasImports;
 
-    void *_private;	/* unused by the library for users or bindings */
+    void *_private;        /* unused by the library for users or bindings */
     xmlHashTablePtr groupDecl;
     xmlDictPtr      dict;
     void *includes;     /* the includes, this is opaque for now */
-    int preserve;	/* whether to free the document */
+    int preserve;        /* whether to free the document */
+    int counter; /* used to give ononymous components unique names */
+    xmlHashTablePtr idcDef;
+    void *volatiles; /* Misc. helper items (e.g. reference items) */    
 };
 
-XMLPUBFUN void XMLCALL 	xmlSchemaFreeType	(xmlSchemaTypePtr type);
-XMLPUBFUN void XMLCALL 	xmlSchemaFreeWildcard(xmlSchemaWildcardPtr wildcard);
+XMLPUBFUN void XMLCALL         xmlSchemaFreeType        (xmlSchemaTypePtr type);
+XMLPUBFUN void XMLCALL         xmlSchemaFreeWildcard(xmlSchemaWildcardPtr wildcard);
 
 #ifdef __cplusplus
 }
@@ -789,4 +903,3 @@ XMLPUBFUN void XMLCALL 	xmlSchemaFreeWildcard(xmlSchemaWildcardPtr wildcard);
 
 #endif /* LIBXML_SCHEMAS_ENABLED */
 #endif /* __XML_SCHEMA_INTERNALS_H__ */
-
